package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.DateConverter;
import de.xoev.xdomea.AntragsdatenFieldType;
import de.xoev.xdomea.AntragsdatenGroupType;
import de.xoev.xdomea.AntragsdatenItemType;
import de.xoev.xdomea.AntragsdatenMultiValueFieldType;
import de.xoev.xdomea.AntragsdatenSingleValueFieldType;
import de.xoev.xdomea.DatatypeType;

class FormDataMapperTest {

	@Spy
	@InjectMocks
	private FormDataMapper formDataMapper;

	@Mock
	private DatatypeMapper datatypeMapper;
	@Mock
	private DateConverter dateConverter;

	@Nested
	class TestToAntragsdaten {

		@Mock
		private VorgangWithEingang vorgang;

		@Test
		void shouldCallBuildAntragsdaten() {
			try (MockedStatic<FormDataExportAdjuster> valueKeysRemover = mockStatic(FormDataExportAdjuster.class)) {
				var vorgang = VorgangWithEingangTestFactory.create();
				Map<String, Object> formDataWithoutValueKeys = Map.of(UUID.randomUUID().toString(), "value1");
				valueKeysRemover.when(() -> FormDataExportAdjuster.adjustFormDataForExport(any())).thenReturn(formDataWithoutValueKeys);
				doReturn(new Object()).when(formDataMapper).formatValue(any(), any());

				formDataMapper.toAntragsdaten(vorgang);

				verify(formDataMapper).buildAntragsdaten(formDataWithoutValueKeys);
			}
		}

		@Test
		void shouldMapEingangNull() {
			var Antragsdaten = formDataMapper.toAntragsdaten(vorgang);

			assertThat(Antragsdaten.getItem()).isEmpty();
		}

		@Test
		void shouldMapFormDataNull() {
			var eingang = mock(Eingang.class);
			when(vorgang.getEingang()).thenReturn(eingang);

			var Antragsdaten = formDataMapper.toAntragsdaten(vorgang);

			assertThat(Antragsdaten.getItem()).isEmpty();
		}

	}

	@Nested
	class TestBuildAntragsdaten {

		private final Map<?,?> formData = EingangTestFactory.create().getFormData();
		private final List<AntragsdatenItemType> items = AntragsdatenItemTypeTestFactory.createAntragsdatenItemTypes();

		@BeforeEach
		void init() {
			doReturn(items).when(formDataMapper).buildItems(any());
		}

		@Test
		void shouldCallBuildItems() {
			formDataMapper.buildAntragsdaten(formData);

			verify(formDataMapper).buildItems(formData);
		}

		@Test
		void shouldFillItems() {
			var antragsdaten = formDataMapper.buildAntragsdaten(formData);

			assertThat(antragsdaten.getItem()).containsExactlyElementsOf(items);
		}
	}

	@Nested
	class TestBuildItems {
		@Mock
		private AntragsdatenItemType expectedAntragsdatenItem;

		@Test
		void shouldReturnEmptyList() {
			var items = formDataMapper.buildItems(Collections.emptyMap());

			assertThat(items).isEmpty();
		}

		@Test
		void shoulAddSubmap() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildAntragsdatenGroup(any(), any());

			var items = formDataMapper.buildItems(Map.of("key", Map.of("key", "value")));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}

		@Test
		void shouldAddCollection() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildMultiValueField(any(), any());

			var items = formDataMapper.buildItems(Map.of("key", List.of("value")));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}

		@Test
		void shouldAddSingleValue() {
			doReturn(expectedAntragsdatenItem).when(formDataMapper).buildSingleValueField(any(), any());

			var items = formDataMapper.buildItems(Map.of("key", "value"));

			assertThat(items).hasSize(1).first().isEqualTo(expectedAntragsdatenItem);
		}
	}

	@Nested
	class TestBuildAntragsdatenGroup {

		private static final String NAME = "name";
		private static final Map<String, Object> FORMDATA_SUBMAP = Map.of("key", "value");
		private final List<AntragsdatenItemType> items = AntragsdatenItemTypeTestFactory.createAntragsdatenItemTypes();

		@Test
		void shouldSetName() {
			doReturn(new Object()).when(formDataMapper).formatValue(any(), any());

			var antragsdatenGroup = formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			assertThat(antragsdatenGroup.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetItems() {
			doReturn(items).when(formDataMapper).buildItems(any());

			var antragsdatenGroup = (AntragsdatenGroupType) formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			assertThat(antragsdatenGroup.getItem()).containsExactlyElementsOf(items);
		}

		@Test
		void shouldCallBuildItems() {
			doReturn(new Object()).when(formDataMapper).formatValue(any(), any());

			formDataMapper.buildAntragsdatenGroup(NAME, FORMDATA_SUBMAP);

			verify(formDataMapper).buildItems(FORMDATA_SUBMAP);
		}
	}

	@Nested
	class TestBuildMultiValueField {

		private static final String NAME = "name";
		private static final List<String> FORMDATA_COLLECTION = List.of("value1");

		@Test
		void shouldSetName() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.STRING);

			var antragsdatenMultiValueFieldType = formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			assertThat(antragsdatenMultiValueFieldType.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetType() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.STRING);

			var antragsdatenMultiValueFieldType = (AntragsdatenFieldType) formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			assertThat(antragsdatenMultiValueFieldType.getType()).isEqualTo(DatatypeType.STRING);
		}

		@Test
		void shouldSetValue() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.INTEGER);
			var formData = List.of(1, 2);

			var antragsdatenMultiValueFieldType = (AntragsdatenMultiValueFieldType) formDataMapper.buildMultiValueField(NAME, formData);

			assertThat(antragsdatenMultiValueFieldType.getValue()).containsExactlyElementsOf(formData);
		}

		@Test
		void shouldCallFormatValue() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.STRING);

			formDataMapper.buildMultiValueField(NAME, FORMDATA_COLLECTION);

			verify(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_COLLECTION.get(0));
		}
	}

	@Nested
	class TestBuildSingleValueField {

		private static final String NAME = "name";
		private static final String FORMDATA_VALUE = "value";

		@Test
		void shouldSetName() {
			doReturn(new Object()).when(formDataMapper).formatValue(any(), any());

			var antragsdatenSingleValueFieldType = formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getName()).isEqualTo(NAME);
		}

		@Test
		void shouldSetType() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.STRING);

			var antragsdatenSingleValueFieldType = (AntragsdatenFieldType) formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getType()).isEqualTo(DatatypeType.STRING);
		}

		@Test
		void shouldSetValue() {
			doReturn(FORMDATA_VALUE).when(formDataMapper).formatValue(any(), any());

			var antragsdatenSingleValueFieldType = (AntragsdatenSingleValueFieldType) formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			assertThat(antragsdatenSingleValueFieldType.getValue()).isEqualTo(FORMDATA_VALUE);
		}

		@Test
		void shouldNotSetIfValueIsNull() {
			formDataMapper.buildSingleValueField(NAME, null);

			verify(formDataMapper, never()).formatValue(DatatypeType.STRING, null);
		}


		@Test
		void shouldCallFormatValue() {
			when(datatypeMapper.from(any())).thenReturn(DatatypeType.STRING);

			formDataMapper.buildSingleValueField(NAME, FORMDATA_VALUE);

			verify(formDataMapper).formatValue(DatatypeType.STRING, FORMDATA_VALUE);
		}
	}

	@Nested
	class TestFormatValue {

		@Test
		void shouldFormatStringType() {
			var value = formDataMapper.formatValue(DatatypeType.STRING, 1);

			assertThat(value).isEqualTo("1");
		}

		@Test
		void shouldFormatDateTimeType() {
			var dateTime = ZonedDateTime.now();

			formDataMapper.formatValue(DatatypeType.DATETIME, dateTime);

			verify(dateConverter).toXmlGregorianCalendar(dateTime);
		}

		@Test
		void shouldFormatNull() {
			var value = formDataMapper.formatValue(DatatypeType.STRING, null);

			assertThat(value).isEqualTo("null");
		}
	}
}